#! /bin/sh


#
# systeminfo.sh - System information gathering script
# by Kirk Waingrow
#
# V1.4-changed memory check
#
# SYNTAX: ./systeminfo.sh [OPTION] 
#
#
version="1.4 - 4/17/2006" 
HOMEPAGE="http://www.ugu.com/systeminfo"

HST=`hostname -s`
DPATH="/usr/local/systeminfo"

dir_exist()
{
if [ ! -d "$DPATH/$HST" ]; then
  mkdir -p $DPATH/$HST
fi
}

display_syntax()
{
  echo "systeminfo.sh [OPTION]"
  echo 
  echo "Only one option is allowed at a time:"
  echo "-a : Display to stdout, generate web pages and tar file."
  echo "-d : Display to stdout ONLY."
  echo "-h : Display this message."
  echo "-t : Creates just a tar file of the web pages."
  echo "-w : Create web pages and master info file ONLY."
  echo "Default (no options) will execute all the above."
  echo
  echo "Data collection can takes about 60 seconds.."
  echo
  exit 0
}

if [ `whoami` != "root" ]; then
  echo
  echo "You need to be 'root' to run this..."
  echo
  exit 1
fi

general_p1()
{
echo "Current System Information as of `date`"
echo
echo "Hostname: `hostname -s`"
echo "Domain: `hostname -d`"
echo "Host Alias: `hostname -a`"
echo "YP Domain: `hostname -d`"
echo "hostID: `hostid`"
echo
echo "Uptime : `uptime `"
}


network_base()
{
echo
for i in `ifconfig | grep "Link " | awk '{ print $1 }' | grep -v lo`
do
  echo "Network $i IP: `ifconfig $i | awk 'NR==2 { print $2}'|awk -F":" '{print $2}'`"
if [ -s "/sbin/mii-tool" ]; then
  echo "Network $i Setting:`/sbin/mii-tool | grep $i: | cut -d':' -f2-`"
fi
  echo "Network $i Netmask: `ifconfig $i | awk 'NR==2 { print $4}'|awk -F":" '{print $2}'`"
  echo "Network $i Hardware Addr: `ifconfig $i | awk 'NR==1 { print $5}'`"
  echo "Network $i txqueuelen: `ifconfig $i | awk 'NR==7 { print $2}'|awk -F":" '{print $2}'`"
  echo
done 
echo "Default Route Interface: `route | grep default | awk '{ print $8 }'`"
echo "Default Route IP: `route | grep default | awk '{ print $2 }'`"

DNSSERVER=`nslookup -sil host | grep "Server:" | awk '{ print $2}'`
if [ -n "$DNSSERVER" ]; then
  SNAME=`nslookup -sil $DNSSERVER | grep "name =" | awk -F'=' '{ print $2}'`
  
  echo "DNS Server: $DNSSERVER"
  echo "DNS Server Name: $SNAME"
fi 
}


general_p2()
{
echo
echo "OS Distribution Name: `uname -s`"
echo "OS Release: `uname -r`"
found=`uname --help | grep "\-o"`
if [ -n "$found" ]; then
  echo "OS Name: `uname -o`"
fi
echo "Machine Name: `uname -m`"
found=`uname --help | grep "\-i"`
if [ -n "$found" ]; then
  echo "Hardware Platform: `uname -i`"
fi
echo "Libc Name: `rpm -qa | grep -i libc-[0-9] | cut -d'-' -f1`";
echo "Libc Version: `rpm -qa | grep -i libc-[0-9] | cut -d'-' -f2-`";
echo
echo "Processor Type: `uname -p`"
echo "Num Processors: `grep processor /proc/cpuinfo | wc -l |awk '{print $1}'`"
command=`grep processor /proc/cpuinfo | awk '{ print $3}' | tail -1`
echo -n "Hyperthreading is "
if [ $command -gt 1 ]; then
  echo "ON"
else
  echo "OFF"
fi
#echo "Memory: `/usr/bin/procinfo | grep "Mem:" | awk '{ print $2}'`"
echo "Memory: `cat /proc/meminfo | grep MemTotal | awk '{print $2}' | cut -d" " -f1`"
echo "Timezone: `date +%Z`"
echo "Init Level: `grep ":initdefault:" /etc/inittab | cut -d":" -f2`"
}


chk_config()
{
echo; echo
echo "CHECK CONFIG:"
echo "=-=-=-=-=-=-=-=-=-=--=-=-=-=-=-=-=-=-="
chkconfig --list
}

diskinfo()
{
echo
echo
echo "MOUNTED DISKS:"
echo "=-=-=-=-=-=-=-=-=-=--=-=-=-=-=-=-=-=-="
df

echo
echo
echo "FILES SYSTEM TABLE"
echo "=-=-=-=-=-=-=-=-=-=--=-=-=-=-=-=-=-=-="
cat /etc/fstab

##
## ASSUMES Disks are ext3 and sd|hd|cciss|ida
##
echo; echo
echo "DISK PARTITION TABLE"
echo "=-=-=-=-=-=-=-=-=-=--=-=-=-=-=-=-=-=-="
for i in `mount -l | grep ext3 | cut -d" " -f1 `
do
  found=`echo $i | egrep '(cciss|ida)'`
  if [ -n "$found" ]; then
    D=`echo $i | cut -d"p" -f1`
    fdisk -l $D
  fi
  found=`echo $i | egrep '(sd|hd)'`
  if [ -n "$found" ]; then
    D=`echo $i | cut -c1-8`
    fdisk -l $D
  fi
done

echo; echo
echo "SWAP:"
echo "=-=-=-=-=-=-=-=-=-=--=-=-=-=-=-=-=-=-="
swapon -s

echo
echo
echo "IOSTAT DISK DEVICES (4 @ 3/sec intervals)"
echo "=-=-=-=-=-=-=-=-=-=--=-=-=-=-=-=-=-=-=-=-="
iostat -t -d -x 3 4
}

networkinfo()
{
echo; echo
echo "NETWORK INTERFACE INFORMATION"
echo "=-=-=-=-=-=-=-=-=-=--=-=-=-=-=-=-=-=-="
ifconfig

echo; echo
echo "DNS RESOLVER - /etc/resolv.conf"
echo "=-=-=-=-=-=-=-=-=-=--=-=-=-=-=-=-=-=-="
cat /etc/resolv.conf

echo; echo
echo "ROUTING TABLE"
echo "=-=-=-=-=-=-=-=-=-=--=-=-=-=-=-=-=-=-="
route
}


systeminfo()
{


echo; echo
echo "KERNEL MODULES"
echo "=-=-=-=-=-=-=-=-=-=--=-=-=-=-=-=-=-=-="
lsmod

echo ;echo
echo "KERNEL TUNABLE PARAMETERS"
echo "=-=-=-=-=-=-=-=-=-=--=-=-=-=-=-=-=-=-="
sysctl -a;

echo; echo
echo "SYSTEM RESOURCE LIMITATIONS - ULIMIT "
echo "=-=-=-=-=-=-=-=-=-=--=-=-=-=-=-=-=-=-="
ulimit -a

echo ; echo
echo "EXTRA CPU INFORMATION"
echo "=-=-=-=-=-=-=-=-=-=--=-=-=-=-=-=-=-=-="
cat /proc/cpuinfo

echo ; echo
echo "EXTRA MEMORY INFORMATION"
echo "=-=-=-=-=-=-=-=-=-=--=-=-=-=-=-=-=-=-="
cat /proc/meminfo

echo ; echo
echo "PROC INFO"
echo "=-=-=-=-=-=-=-=-=-=--=-=-=-=-=-=-=-=-="
/usr/bin/procinfo -a

echo; echo
echo "VMSTAT (4 @ 3sec interval)"
echo "=-=-=-=-=-=-=-=-=-=--=-=-=-=-=-=-=-=-="
vmstat -n 3 5

echo
echo
echo "IOSTAT CPU (4 @ 3/sec intervals)"
echo "=-=-=-=-=-=-=-=-=-=--=-=-=-=-=-=-=-=-="
iostat -t -c -x 3 4
}


sockets()
{
echo; echo
echo "LISTENING SOCKET PORTS"
echo "=-=-=-=-=-=-=-=-=-=--=-=-=-=-=-=-=-=-="
netstat -an | grep LISTEN
}


software()
{
echo; echo
echo "SOFTWARE INFORMATION"
echo "=-=-=-=-=-=-=-=-=-=--=-=-=-=-=-=-=-=-="
rpm -qa --queryformat '%-15{NAME} %{VERSION}-%-15{RELEASE} %-20{SUMMARY}\n' | sort
}


croninfo()
{
list=`cd /var/spool/cron; ls -1`
for i in $list
do
  if [ -s "/var/spool/cron/$i" ]; then
   echo
   echo
   echo "CRONTAB ENTRY - $i"
   echo "=-=-=-=-=-=-=-=-=-=--=-=-=-=-=-=-=-=-="
   echo
   cat /var/spool/cron/$i
  fi
done
}


proctable()
{
echo; echo
echo "LAST RUN TOP"
echo "=-=-=-=-=-=-=-=-=-=--=-=-=-=-=-=-=-=-="
top -bn 1

echo; echo
echo "LAST RUN PROCESS TABLE"
echo "=-=-=-=-=-=-=-=-=-=--=-=-=-=-=-=-=-=-="
ps -efw
}

dmesginfo()
{
echo; echo
echo "DMESG"
echo "=-=-=-=-=-=-=-=-=-=--=-=-=-=-=-=-=-=-="
dmesg
echo
echo
echo
echo "=-=-=-=-=-=-=-=-=-=--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-"
echo "systeminfo.sh - $version - $HOMEPAGE";echo;echo
}

indexpage()
{
 cat << EOF
	<HTML>
	<title>$HST System Information</title>
	<FRAMESET cols="20%, 80%">
      	<FRAME src="./menu.html" name="menu">
  	<FRAME src="./general.txt" name="primary">
	</frameset>
	</HTML> 

EOF
}


menupage()
{
  cat << EOF

	<title>Menu</title>
	<a href=".." target="_top"><<</a><br>
	<center><b>$HST</b><br>System Info Menu</center><br><hr><p>
   	<A href=$HST.txt target=primary>Show All</a><p>
   	<A href=general.txt target=primary>General Info</a><br>
   	<A href=system.txt target=primary>System Configs</a><br>
   	<A href=disks.txt target=primary>Disks</a><br>
   	<A href=network.txt target=primary>Network</a><br>
   	<A href=software.txt target=primary>Software Packages</a><br>
   	<A href=cron.txt target=primary>Cron Jobs</a><br>
   	<A href=sockets.txt target=primary>Socket Ports</a><br>
   	<A href=proc.txt target=primary>Top/Proc Table</a><br>
   	<A href=chkconfig.txt target=primary>chkconfig</a><br>
   	<A href=dmesg.txt target=primary>dmesg</a><br>
	<pre>

	</pre>
	<hr>
	<font size=2>Version: $version</font><p>
EOF

}


display_update()
{
  if [ "$OPT" = "-t" -o "$OPT" = "-w" ]; then 
    echo "$msg"; 
  fi
}



###
### Gather Information
###
build_file()
{
  general_p1
  network_base
  general_p2	
  chk_config	
  diskinfo
  networkinfo
  systeminfo	
  sockets	
  software	
  croninfo	
  proctable	
  dmesginfo
}


###
### GATHER AND GENERATE PAGES
###
generate_pages()
{
  indexpage	> $DPATH/$HST/index.html
  menupage	> $DPATH/$HST/menu.html
  msg="General Info"; display_update
  general_p1 	> $DPATH/$HST/general.txt
  msg="Network Info"; display_update
  network_base 	>> $DPATH/$HST/general.txt
  msg="More General Info"; display_update
  general_p2 	>> $DPATH/$HST/general.txt	
  msg="Config Info"; display_update
  chk_config	> $DPATH/$HST/chkconfig.txt	
  msg="Disk Info"; display_update
  diskinfo	> $DPATH/$HST/disks.txt	
  msg="More Network Info"; display_update
  networkinfo	> $DPATH/$HST/network.txt
  network_base	>> $DPATH/$HST/network.txt
  msg="System Info"; display_update
  systeminfo	> $DPATH/$HST/system.txt	
  msg="Socket Info"; display_update
  sockets	> $DPATH/$HST/sockets.txt
  msg="Software Info"; display_update
  software	> $DPATH/$HST/software.txt
  msg="Cron Info"; display_update
  croninfo	> $DPATH/$HST/cron.txt
  msg="Processor Info"; display_update
  proctable	> $DPATH/$HST/proc.txt
  msg="Dmesg Info"; display_update
  dmesginfo	> $DPATH/$HST/dmesg.txt

cd $DPATH/$HST
cat general.txt chkconfig.txt disks.txt network.txt system.txt sockets.txt software.txt cron.txt proc.txt dmesg.txt > $DPATH/$HST/$HST.txt

}

generate_tar()
{
  echo "Building Tar file"
  cd $DPATH
  tar cf systeminfo_$HST.tar $HST
}


echo "Version $version - $HOMEPAGE"; 
if [ -n "$1" ];then
  OPT="$1"
  case $1 in 
	  # Display stdout, generate web pages and tar file
    "-a") dir_exist 
	  echo "Generating stdout, Web Pages and Tar File - Please wait..."
  	  generate_pages
  	  generate_tar
  	  cat $DPATH/$HST/$HST.txt
  	  echo; 
  	  echo "Master Info File: $DPATH/$HST/$HST.txt"
  	  echo "HTML Start Page : $DPATH/$HST/index.html"
  	  echo "Tar File        : $DPATH/systeminfo_$HST.tar"
	  ;;

          # Display to stdout
    "-d") echo;echo;build_file;; 

          # Display help/syntax.
    "-h") display_syntax ;; 

	  # Creates just a tar file of the webpages.
    "-t") dir_exist; 
	  echo "Building Tar File - Collecting..."
	  generate_pages
	  generate_tar
	  cd $DPATH/$HST
          rm general.txt chkconfig.txt disks.txt network.txt system.txt \
	  sockets.txt software.txt cron.txt proc.txt dmesg.txt $HST.txt \
	  index.html menu.html
	  rmdir $DPATH/$HST  
  	  echo; echo "Tar File: $DPATH/systeminfo_$HST.tar"; echo
	  ;;

	  # Create web pages and master info file. 
    "-w") dir_exist; 
	  echo "Building Web Pages - Collecting..."
	  generate_pages
	  echo; echo; echo "Master Info File: $DPATH/$HST/$HST.txt"
	  echo; echo "HTML Start Page: $DPATH/$HST/index.html"; echo 
	  ;;

          # Display help/syntax.
       *) display_syntax;;
  esac
else
  # Display to stdout (same as -d)
  echo; echo; 
  build_file
fi
exit 0
